// Copyright 2008 Serkan Cabi
// This file is part of Kognite.
// 
// Kognite is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Kognite is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Kognite.  If not, see <http://www.gnu.org/licenses/>.

#include <QtCore>
#include <QtGui>
#include <xapian.h>
#include <poppler-qt4.h>

#ifdef Q_WS_MAC
#include <qtspellcheckingtextedit.h>
#endif

using namespace std;

class MyTreeView : public QTreeView
	{
		Q_OBJECT
		
		private slots:
		
		void dropEvent(QDropEvent *);
		
	signals:
		
		void fileMoveRequested(QString, QPoint);
		void directoryMoveRequested(QString, QPoint);
		;
	};

class MainWindow : public QMainWindow
	{
		Q_OBJECT
		
		public slots:
		
		void processTextChange();
		void setCurrentFileName(const QString&);
		bool load(const QString&);
		void loadClicked();
		void loadResult();
		void openExternally();
		bool loadConfigFile();
		void updateConfigFile();
		void setupMenuActions();
		void closeEvent(QCloseEvent *);
		void showFindBox();
		void hideFindBox();
		void showReplaceRow();
		void showFindAndReplace();
		void keyPressEvent(QKeyEvent *);
		void findNext();
		void findPrev();
		void findWithNewOptions();
		bool find(QString, bool forward = false, bool backward = false);
		void replaceWith();
		void replaceAll();
		void disableReplace();
		void loadDirectory();
		void refreshFilesTree();
		void refreshDirsTree();
		void prepareDirsTree();
		void prepareLayout();
		void prepareToolBar();
		void moveDirectory(QString, QPoint);
		void moveFile(QString, QPoint);
		void deleteCurrentFile();
		void renameFileStart(QModelIndex);
		void renameFileEnd(QWidget *);
		void renameDirStart(QModelIndex);
		void renameDirEnd(QWidget *);
		void dirAdd();
		void dirDelete();
		bool dirDeleter(QDir);
		void goHome();
		
		bool fileSave();
		bool fileSaveAs();
		void newPlainFile();
		void newRichFile();	
		bool maybeSave();
		void filePrint();
		void filePrintPdf();
		void toggleToolBar();
		void toggleMiddleView();
		void resetView();
		void refreshAll();
		void crop();
		void about();
		void chooseFilesPath(bool shouldIndex = true);
		void chooseDbPath(bool shouldIndex = true);
		void textBold();
		void textUnderline();
		void textItalic();
		void textColor();
		void toggleFullScreen();
		void textAlign(QAction *a);
		void mergeFormatOnWordOrSelection(const QTextCharFormat&);
		void setRichTextActions(bool);
		void currentCharFormatChanged(const QTextCharFormat&);
		void cursorPositionChanged();	
		void fontChanged(const QFont&);
		void alignmentChanged(Qt::Alignment);
		
		void loadPdf(QString pdfFile);
		void pdfZoom(double pdfZoomFactor);
		void pdfZoomIn();
		void pdfZoomOut();
		void clearPdfCache();
		bool pdfIndexer(QString file2IndexPath);
		
		bool indexer();
		int fileCounter(QDir);
		bool dirIndexer(QDir);
		bool fileIndexer(QString);
		void searcher();
		Xapian::docid idOfDocument(QString);
		bool getSimilar();
		void renameInDB(QString, QString);
		void renameDirInDB(QString, QString);
		void removeFromDB(QString);
		void removeDirFromDB(QString);
		
	public:
		MainWindow(QWidget *parent = 0);
		
#ifdef Q_WS_MAC
		QtSpellCheckingTextEdit *textEdit;
#else
		QTextEdit *textEdit;
#endif
		
		QSettings *settings;
		
		int messageTime;
		int nameColumnWidth;
		bool initialLaunch;
		bool inFullScreen;
		bool searchResults;
		int defaultTextFrameStyle;
		int defaultPdfFrameStyle;
		int pdfWidth;
		bool documentLoading;
		double pdfResolution;
		
		QAction *a;
		QAction *actionNewFile;
		QAction *actionSave;
		QAction *actionPrint;
		QAction *actionUndo;
		QAction *actionRedo;
		QAction *actionCut;
		QAction *actionCopy;
		QAction *actionPaste;
		QAction *actionSelectAll;	
		QAction *actionCrop;
		QAction *actionFind;
		QAction *actionFindNext;
		QAction *actionFindPrev;
		QAction *actionFindAndReplace;
		QAction *actionZoomIn;
		QAction *actionZoomOut;
		QAction *actionToolBar;
		QAction *actionMiddleView;
		QAction *actionTextBold;
		QAction *actionTextItalic;
		QAction *actionTextUnderline;
		QAction *actionAlignLeft;
		QAction *actionAlignCenter;
		QAction *actionAlignRight;
		QAction *actionAlignJustify;
		QAction *actionTextColor;
		QAction *actionFullScreen;
		QAction *tbFind;
		QAction *tbZoomIn;
		QAction *tbZoomOut;
		QAction *tbFulScreen;
		
		QToolBar *tb;
		QToolBar *findBar;
		QToolBar *rightBar;	
		QMenu *menu;
		QString filesPath;
		QString databasePath;
		QString currentDirPath;
		QString fileName;
		QString file2Rename;
		QString dir2Rename;
		QString forcedExtension1;
		QString forcedExtension2;
		MyTreeView *filesTree;
		MyTreeView *dirsTree;
		QGraphicsView *pdfViewer;
		QGraphicsScene *pdfScene;
		QList<QGraphicsPixmapItem*> pdfPageItems;
		QFrame *cantOpenWidget;
		QLabel *cantOpenLabel;
		QVBoxLayout *cantOpenLayout;
		QPushButton *openExternallyButton;
		QFrame *DocumentViewer;
		QFrame *leftFrame;
		QFrame *rightFrame;
		QFrame *fsFrame;
		QSplitter *middleFrame;
		QSplitter *central;
		QGridLayout *searchLayout;
		QGridLayout *leftLayout;
		QGridLayout *rightLayout;
		QGridLayout *DocumentViewerLayout;
		QFrame *findFrame;
		QGridLayout *fsLayout;
		QWidget *fsCentral;
		QGridLayout *findLayout;
		QLineEdit *findBox;
		QLineEdit *replaceBox;
		QCheckBox *checkCase;
		QCheckBox *checkWholeWords;
		QDirModel *filesModel;
		QDirModel *dirsModel;
		QModelIndex parentDirIndex;
		QGroupBox *dirsContainer;
		QGroupBox *searchContainer;
		QGroupBox *resultsContainer;
		QListWidget *resultsList;
		QLineEdit *searchBox;
		QPushButton *similarsButton;
		QPushButton *replaceWithButton;
		QPushButton *replaceButton;
		QPushButton *replaceAllButton;
		QDialog *dialogBox;
		QProgressDialog progress;
		QProgressBar *indexProgress;
		QList<int> middleSizes;
		QList<int> horizontalSizes;
		QRect winGeo;
	};

